#
# admin_cgi.py - Admin CGI methods.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: admin_cgi.py,v 1.28 2005/10/28 13:28:53 fuktommy Exp $
#

import cgi
import re

import config
import gateway
from cache import *
from node import *

__version__ = "$Revision: 1.28 $"


class CGI(gateway.CGI):

    """Class for /admin.cgi."""

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)

        if not re.search(config.admin, self.environ["REMOTE_ADDR"]):
            self.print403()
            return
        elif form.getfirst("cmd", "") == "delete":
            if form.getfirst("file", "") == "":
                self.print404()
            elif form.getfirst("record", "") == "":
                self.print_delete_file(form.getfirst("file", ""))
            else:
                self.print_delete_record(form.getfirst("file", ""),
                                         form.getfirst("record", ""))
        elif form.getfirst("cmd", "") == "xdelete" and \
             self.check_referer() and \
             self.environ["REQUEST_METHOD"] == "POST":
            if form.getfirst("file", "") == "":
                self.print404()
            elif form.getfirst("record", "") == "":
                self.do_delete_file(form.getfirst("file", ""))
            else:
                self.do_delete_record(form.getfirst("file", ""),
                                      form.getfirst("record", ""),
                                      form.getfirst("dopost", ""),
                                      form)
        elif path == "index":
            self.print_index()
        elif path == "changes":
            self.print_changes()
        elif path == "update":
            self.print_update()
        elif path.startswith("search"):
            self.print_search(path, form)
        elif path.startswith("status"):
            self.print_status()
        else:
            self.print404()

    def check_referer(self):
        """Check referer is delete form page or not."""
        referer = self.environ.get("HTTP_REFERER", "")
        found = re.search(r"http://([^/]+)/.*delete", referer)
        if found:
            host = found.group(1)
            return host == self.environ.get("HTTP_HOST", "")
        else:
            return False

    def print_delete_record(self, datfile, idstr):
        """Delete record dialog."""
	self.header(self.message["del_record"], deny_robot=True)
        rec = Record(datfile=datfile, idstr=idstr)
        if not rec.exists():
            self.stdout.write("<p>" + self.message["no_record"] + "</p>")
        else:
            self.print_delete_record_form(datfile, rec)
        self.footer()

    def print_delete_record_form(self, datfile, rec):
        rec.load()
        buf = re.sub(r"attach:[^<>]+", "attach:1", rec.recstr)
        buf = cgi.escape(buf)
        self.stdout.write(
            "<p>" + self.message["del_record_q"] + "</p>" +
            '<form method="post" action="' + self.admin_cgi + '"><p>' +
            '<input type="hidden" name="cmd" value="xdelete" />' +
            '<input type="hidden" name="file" value="' + rec.datfile + '" />' +
            '<input type="hidden" name="record" value="' + rec.idstr + '" />' +
            '<input type="submit" name="submit" value="' +
            self.message["yes"] + '" name="submit"' +
            ' tabindex="1" accesskey="w" />' +
            ' <input type="checkbox" name="dopost" value="dopost"' +
            ' tabindex="2" accesskey="s" />' + 
            self.message["send"] + "<br />" +
            " " + self.message["name"] +
            ':<input name="name" size="15" value=""' +
            ' tabindex="3" accesskey="n" />' +
            " " + self.message["signature"] +
            ':<input type="password" name="passwd" size="15"' +
            ' value="" tabindex="4" accesskey="p" />' +
            " " + self.message["comment"] + 
            ':<input name="message" size="15" value=""' +
            ' tabindex="5" accesskey="m" />' +
            "</p></form>\n" +
            "<p>" + buf + "</p>\n")

    def do_delete_record(self, datfile, idstr, dopost="", form=None):
        rec = Record(datfile=datfile, idstr=idstr)
        size = rec.size()
        removed = rec.remove()
        cache = Cache(datfile)
        cache.sync_body_light(size=size)
        title = self.str_encode(self.file_decode(datfile))
        for type in config.types:
            if datfile.startswith(type + "_"):
                next = self.appli[type] + self.sep + title
                break
        else:
            next = self.root

        if removed and (dopost != ""):
            self.post_delete_message(cache, rec, form)

        self.print302(next)

    def post_delete_message(self, cache, rec, form):
        """Post delete message to other nodes."""
        stamp = self.error_time()

        body = {}
        for key in ("name", "message"):
            if form.getfirst(key, "") != "":
                body[key] = self.escape(form.getfirst(key, ""))
        body["remove_stamp"] = str(rec.stamp)
        body["remove_id"] = rec.id
        passwd = form.getfirst("passwd", "")
        id = rec.build(stamp, body, passwd=passwd)
        cache.add_data([rec])

        updatelist = UpdateList()
        updatelist.append(rec)
        updatelist.sync()
        nodelist = NodeList()
        nodelist.tell_update(cache, stamp=stamp, id=id)

    def print_delete_file(self, datfile):
        """Delete file dialog."""
        cache = Cache(datfile)
	self.header(self.message["del_file"], deny_robot=True)
        if cache.exists():
            self.print_delete_file_form(cache)
        else:
            self.stdout.write("<p>%s: %s</p>" %
                              (datfile, self.message["no_file"]))
        self.footer()

    def print_delete_file_form(self, cache):
        self.stdout.write(
            '<form method="post" action="' + self.admin_cgi + '"><p>\n' +
            "  " + self.message["del_file_q"] + "\n" +
            '  <input type="hidden" name="cmd" value="xdelete" />\n' +
            '  <input type="hidden" name="file" ' +
            'value="' + cache.datfile + '" />\n' +
            '  <input type="submit" value="%s" tabindex="1" accesskey="y" />\n'
            % self.message["yes"] +
            '</p></form>\n')
        for type in config.types:
            if cache.datfile.startswith(type + "_"):
                mode = self.message[type]
                title = self.file_decode(cache.datfile)
                break
        else:
            mode = ""
            title = cache.datfile
        self.stdout.write("<p>%s(%s)</p>\n" % (title, mode))
        count = 0
        for idstr in cache:
            count += 1
            rec = cache[idstr]
            rec.load()
            buf = re.sub(r"attach:[^<>]+", "attach:1", rec.recstr)
            buf = cgi.escape(buf)
            self.stdout.write("<p>%s</p>" % buf)
            if (count > 2):
                break

    def do_delete_file(self, datfile):
        cache = Cache(datfile)
        cache.remove()
        self.print302(self.admin_cgi + self.sep + "changes")

    def print_index_list(self, cachelist, footer=True):
        """Print index list."""
        self.stdout.write(
            '<form method="get" action="%s">\n' % self.admin_cgi +
            '<p><input type="hidden" name="cmd" value="delete" /></p>\n')
        for type in config.types:
            self.stdout.write(
                "<h2>" + self.message[type] + "</h2>\n" +
                "<ul>\n")
                
            for cache in cachelist:
                if cache.type != type:
                    continue
                x = self.file_decode(cache.datfile)
                y = self.str_encode(x)
                self.stdout.write(
                    "  <li>\n" +
                    '    <input type="radio" name="file" value="' +
                    cache.datfile + '" tabindex="1" accesskey="s" />\n' +
                    self.localtime(cache.stamp) + " " +
                    '    <a href="' + self.appli[type] + self.sep +
                    y + '">' + x + '</a>\n' +
                    '    (' + str(len(cache)) + "/" +
                    str(int(cache.size/1024/1024*10)/10.0) +
                    self.message["mb"] + ")\n" +
                    "  </li>\n")
            self.stdout.write("</ul>\n")
        self.stdout.write(
            '<p><input type="submit" value="' +
            self.message["del_file"] + '" tabindex="2" accesskey="d" />' +
            '</p>\n' +
            "</form>\n")
        if footer:
            self.print_new_element_form();
            self.footer()

    def print_index(self):
        """Print index page."""
        self.header(self.message["index"])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(a.datfile, b.datfile))
        self.print_index_list(cachelist)

    def print_changes(self):
        """Print changes page."""
        self.header(self.message["changes"])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        self.print_index_list(cachelist)

    def print_update(self):
        """Print changes page."""
        self.header(self.message["update"])
        updatelist = UpdateList()
        updatelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        cachelist = []
        check = []
        for rec in updatelist:
            if rec.datfile not in check:
                cache = Cache(rec.datfile)
                cache.stamp = rec.stamp
                cachelist.append(cache)
                check.append(rec.datfile)
        self.print_index_list(cachelist)

    def print_search_form(self, query=""):
        self.stdout.write(
            '<form method="get" action="' + self.admin_cgi + '/search"><p>\n'+
            '  <input name="query" size="19" value="' + query + '"' +
            ' tabindex="1" accesskey="q" />\n' +
            '  <input type="submit" value="' + self.message["search"] + '"' +
            ' name="submit" tabindex="1" accesskey="s" />\n' +
            '</p></form>\n')

        self.stdout.write(
            '<form method="get" action="' + config.google + '"><p>\n' +
            '  <input name="q" size="19" value="' + query + '"'
            ' tabindex="1" accesskey="g" />\n' +
            '  <input type="hidden" name="ie" value="UTF-8" />\n' +
            '  <input type="submit" name="google" value="' +
            self.message["google"] + '" tabindex="1" accesskey="g" />\n' +
            "</p></form>\n")

    def print_search_result(self, query):
        self.header(self.message["search"], deny_robot=True)
        cachelist = CacheList().search(query)
        self.print_index_list(cachelist, footer=False)
        self.print_search_form(query)
        self.footer()

    def print_search(self, path, form):
        query = cgi.escape(form.getfirst("query", ""), True)
        if query == "":
            query = path[len("search/"):].replace('"', "&quote;")

        if query == "":
            query = self.environ.get("QUERY_STRING", "")
            query = self.escape(self.str_decode(query))

        if query == "":
            self.header(self.message["search"], deny_robot=True)
            self.print_search_form()
            self.footer()
        else:
            self.print_search_result(query)

    def print_status(self):
        self.header(self.message["status"], deny_robot=True)
        self.stdout.write('<table summary="%s">\n' % self.message["status"])

        nodelist = NodeList()
        searchlist = SearchList()
        cachelist = CacheList()

        records = 0
        size = 0
        for cache in cachelist:
            records += len(cache)
            size += cache.size

        myself = nodelist.myself()

        for p in (("linked_nodes", len(nodelist)),
                  ("known_nodes", len(searchlist)),
                  ("files", len(cachelist)),
                  ("records", records),
                  ("cache_size", str(size*10/1024/1024/10.0) +
                                 self.message["mb"]),
                  ("self_node", myself)):
            self.stdout.write("  <tr><td>%s</td><td>%s</td></tr>\n" %
                              (self.message[p[0]], str(p[1])))

        self.stdout.write("</table>")

        for p in (("linked_nodes", nodelist),
                  ("known_nodes", searchlist)):
            self.stdout.write("<h2>%s</h2>\n<ul>\n" % self.message[p[0]])
            for node in p[1]:
                self.stdout.write("  <li>%s</li>\n" % str(node))
            self.stdout.write("</ul>\n")

        self.footer()
